@@ -9,7 +9,15 @@ use SelectSaver;
 require Net::LDAP::Entry;
 use vars qw($VERSION);
 
-$VERSION = "0.15";
+use constant CHECK_UTF8 => $] > 5.007;
+
+BEGIN {
+  require Encode
+    if (CHECK_UTF8);
+}  
+
+
+$VERSION = "0.17";
 
 my %mode = qw(w > r < a >>);
 
@@ -50,9 +58,11 @@ sub new {
   # Default the error handling to die 
   $opt{'onerror'} = 'die' unless exists $opt{'onerror'};
 
+  # sanitize options
   $opt{'lowercase'} ||= 0;
   $opt{'change'} ||= 0;
   $opt{'sort'} ||= 0;
+  $opt{'version'} ||= 0;
 
   my $self = {
     changetype => "modify",
@@ -62,7 +72,7 @@ sub new {
     fh   => $fh,
     file => "$file",
     opened_fh => $opened_fh,
-    eof => 0,
+    _eof => 0,
     write_count => ($mode eq 'a' and tell($fh) > 0) ? 1 : 0,
   };
 
@@ -77,35 +87,49 @@ sub new {
   
 sub _read_lines {
   my $self = shift;
-  my @ldif;
+  my $fh = $self->{'fh'};
+  my @ldif = ();
+  my $entry = '';
+  my $in_comment = 0;
+  my $entry_completed = 0;
+  my $ln;
 
-  {
-    local $/ = "";
-    my $fh = $self->{'fh'};
-    my $ln;
-    do {	# allow comments separated by blank lines
-      $ln = $self->{_next_lines} || scalar <$fh>;
-      unless ($ln) {
-         $self->{_next_lines} = '';
-         $self->{_current_lines} = '';
-         $self->eof(1);
-         return;
+  return @ldif  if ($self->eof());
+  
+  while (defined($ln = $self->{_buffered_line} || scalar <$fh>)) {
+    delete($self->{_buffered_line});
+    if ($ln =~ /^#/o) {		# ignore 1st line of comments
+      $in_comment = 1;
+    }
+    else {
+      if ($ln =~ /^[ \t]/o) {	# append wrapped line (if not in a comment)
+        $entry .= $ln  if (!$in_comment);
       }
-      $ln =~ s/\n //sg;
-      $ln =~ s/^#.*\n//mg;
-      chomp($ln);
-      $self->{_current_lines} = $ln;
-    } until ($self->{_current_lines} || $self->eof());
-    chomp(@ldif = split(/^/, $ln));
-    do {
-      $ln = scalar <$fh> || '';
-      $self->eof(1) unless $ln;
-      $ln =~ s/\n //sg;
-      $ln =~ s/^#.*\n//mg;
-      chomp($ln);
-      $self->{_next_lines} = $ln;
-    } until ($self->{_next_lines} || $self->eof());
+      else {
+        $in_comment = 0;
+        if ($ln =~ /^\r?\n$/o) {
+          # ignore empty line on start of entry
+          # empty line at non-empty entry indicate entry completion
+          $entry_completed++  if (length($entry));
+	}
+        else {
+	  if ($entry_completed) {
+	    $self->{_buffered_line} = $ln;
+	    last;
+	  }
+	  else {
+            # append non-empty line
+            $entry .= $ln;
+	  }  
+        }	
+      }
+    }
   }
+  $self->eof(1)  if (!defined($ln));
+  $entry =~ s/\r?\n //sgo;	# un-wrap wrapped lines
+  $entry =~ s/\r?\n\t/ /sgo;	# OpenLDAP extension !!!
+  @ldif = split(/^/, $entry);
+  map { s/\r?\n$//; } @ldif;
 
   @ldif;
 }
@@ -120,7 +144,7 @@ sub _read_url_attribute {
 
   if ($url =~ s/^file:(?:\/\/)?//) {
     my $fh = $self->{_attr_fh};
-    unless (open($fh, '<', $url)) {
+    unless (open($fh, '<'.$url)) {
       $self->_error("can't open $line: $!", @ldif);
       return;
     }
@@ -154,11 +178,9 @@ sub _read_entry {
     $self->_error("illegal empty LDIF entry")  if (!$self->eof());
     return;
   }
-  # What does that mean ???
-  #shift @ldif if @ldif && $ldif[0] !~ /\D/;
 
   if (@ldif and $ldif[0] =~ /^version:\s+(\d+)/) {
-    $self->{version} = $1;
+    $self->{'version'} = $1;
     shift @ldif;
     return $self->_read_entry
       unless @ldif;
@@ -175,7 +197,7 @@ sub _read_entry {
 
   my $dn = shift @ldif;
 
-  if (length($1)) {
+  if (length($1)) {	# $1 is the optional colon from above
     eval { require MIME::Base64 };
     if ($@) {
       $self->_error($@, @ldif);
@@ -185,6 +207,8 @@ sub _read_entry {
   }
 
   my $entry = Net::LDAP::Entry->new;
+  $dn = Encode::decode_utf8($dn)
+    if (CHECK_UTF8 && $self->{raw} && ('dn' !~ /$self->{raw}/));
   $entry->dn($dn);
 
   if ($ldif[0] =~ /^changetype:\s*/) {
@@ -215,19 +239,26 @@ sub _read_entry {
       while(@ldif) {
         my $line = shift @ldif;
         my $attr;
+	my $xattr;
   
         if ($line eq "-") {
-          $entry->$modify($lastattr, \@values)
-            if defined $lastattr;
+          if (defined $lastattr) {
+	    if (CHECK_UTF8 && $self->{raw}) {
+  	      map { $_ = Encode::decode_utf8($_) } @values
+	        if ($lastattr !~ /$self->{raw}/);
+	    }  
+            $entry->$modify($lastattr, \@values);
+	  }  
           undef $lastattr;
           @values = ();
           last;
         }
  
-        $line =~ s/^([-;\w]+):\s*// and $attr = $1;
+        $line =~ s/^([-;\w]+):([\<\:]?)\s*// and
+	    ($attr, $xattr) = ($1, $2);
 
         # base64 encoded attribute: decode it
-        if ($line =~ s/^:\s*//) {
+        if ($xattr eq ':') {
           eval { require MIME::Base64 };
           if ($@) {
             $self->_error($@, @ldif);
@@ -236,7 +267,7 @@ sub _read_entry {
           $line = MIME::Base64::decode($line);
         }
         # url attribute: read in file:// url, fail on others
-        elsif ($line =~ s/^\<\s*(.*?)\s*$/$1/) {
+        elsif ($xattr eq '<' and $line =~ s/^(.*?)\s*$/$1/) {
           $line = $self->_read_url_attribute($line, @ldif);
           return  if !defined($line);
         }
@@ -247,16 +278,26 @@ sub _read_entry {
         }
 
         if(!defined($lastattr) || $lastattr ne $attr) {
-          $entry->$modify($lastattr, \@values)
-            if defined $lastattr;
+          if (defined $lastattr) {
+	    if (CHECK_UTF8 && $self->{raw}) {
+  	      map { $_ = Encode::decode_utf8($_) } @values
+	        if ($lastattr !~ /$self->{raw}/);
+	    }  
+            $entry->$modify($lastattr, \@values);
+	  }  
           $lastattr = $attr;
           @values = ($line);
           next;
         }
         push @values, $line;
       }
-      $entry->$modify($lastattr, \@values)
-       if defined $lastattr;
+      if (defined $lastattr) {
+        if (CHECK_UTF8 && $self->{raw}) {
+  	  map { $_ = Encode::decode_utf8($_) } @values
+	    if ($lastattr !~ /$self->{raw}/);
+        }  
+        $entry->$modify($lastattr, \@values);
+      }  
     }
   }
 
@@ -266,11 +307,14 @@ sub _read_entry {
     my $vals = [];
     my $line;
     my $attr;
+    my $xattr;
+
     foreach $line (@ldif) {
-      $line =~ s/^([-;\w]+):\s*// && ($attr = $1) or next;
+      $line =~ s/^([-;\w]+):([\<\:]?)\s*// &&
+	  (($attr, $xattr) = ($1, $2)) or next;
   
       # base64 encoded attribute: decode it
-      if ($line =~ s/^:\s*//) {
+      if ($xattr eq ':') {
         eval { require MIME::Base64 };
         if ($@) {
           $self->_error($@, @ldif);
@@ -279,11 +323,16 @@ sub _read_entry {
         $line = MIME::Base64::decode($line);
       }
       # url attribute: read in file:// url, fail on others
-      elsif ($line =~ s/^\<\s*(.*?)\s*$/$1/) {
+      elsif ($xattr eq '<' and $line =~ s/^(.*?)\s*$/$1/) {
         $line = $self->_read_url_attribute($line, @ldif);
         return  if !defined($line);
       }
   
+      if (CHECK_UTF8 && $self->{raw}) {
+        $line = Encode::decode_utf8($line)
+          if ($attr !~ /$self->{raw}/);
+      }
+
       if ($attr eq $last) {
         push @$vals, $line;
         next;
@@ -335,14 +384,13 @@ sub eof {
 }
 
 sub _wrap {
-  if($_[1] > 40) {
-    my $pos = $_[1];
-    while($pos < length($_[0])) {
-      substr($_[0],$pos,0) = "\n ";
-      $pos += $_[1]+1;
-    }
-  }
-  $_[0];
+  my $len=$_[1];
+  return $_[0] if length($_[0]) <= $len;
+  use integer;
+  my $l2 = $len-1;
+  my $x = (length($_[0]) - $len) / $l2;
+  my $extra = (length($_[0]) == ($l2 * $x + $len)) ? "" : "a*";
+  join("\n ",unpack("a$len" . "a$l2" x $x . $extra,$_[0]));
 }
 
 sub _write_attr {
@@ -351,7 +399,10 @@ sub _write_attr {
   my $res = 1;	# result value
   foreach $v (@$val) {
     my $ln = $lower ? lc $attr : $attr;
-    if ($v =~ /(^[ :]|[\x00-\x1f\x7f-\xff])/) {
+
+    $v = Encode::encode_utf8($v)
+      if (CHECK_UTF8 and Encode::is_utf8($v));
+    if ($v =~ /(^[ :<]|[\x00-\x1f\x7f-\xff])/) {
       require MIME::Base64;
       $ln .= ":: " . MIME::Base64::encode($v,"");
     }
@@ -384,6 +435,9 @@ sub _write_attrs {
 
 sub _write_dn {
   my($dn,$encode,$wrap) = @_;
+
+  $dn = Encode::encode_utf8($dn)
+    if (CHECK_UTF8 and Encode::is_utf8($dn));
   if ($dn =~ /^[ :<]|[\x00-\x1f\x7f-\xff]/) {
     if ($encode =~ /canonical/i) {
       require Net::LDAP::Util;
@@ -417,6 +471,16 @@ sub write_entry {
   $self->_write_entry($self->{change}, @_);
 }
 
+sub write_version {
+  my $self = shift;
+  my $res = 1;
+  
+  $res &&= print "version: $self->{'version'}\n"
+    if ($self->{'version'} && !$self->{version_written}++);
+  
+  return $res;	    
+}
+
 # internal helper: write entry in different format depending on 1st arg
 sub _write_entry {
   my $self = shift;
@@ -449,13 +513,8 @@ sub _write_entry {
       # Skip entry if there is nothing to write
       next if $type eq 'modify' and !@changes;
 
-      if ($self->{write_count}++) {
-	$res &&= print "\n";
-      }
-      else {
-        $res &&= print "version: $self->{version}\n\n"
-          if defined $self->{version};
-      }
+      $res &&= $self->write_version()  unless $self->{write_count}++;
+      $res &&= print "\n";
       $res &&= _write_dn($entry->dn,$self->{'encode'},$wrap);
 
       $res &&= print "changetype: $type\n";
@@ -469,10 +528,10 @@ sub _write_entry {
       }
       elsif ($type =~ /modr?dn/o) {
         my $deleteoldrdn = $entry->get_value('deleteoldrdn') || 0;
-        $res &&= print _write_attr('newrdn',$entry->get_value('newrdn', asref => 1),$wrap,$lower);
+        $res &&= _write_attr('newrdn',$entry->get_value('newrdn', asref => 1),$wrap,$lower);
         $res &&= print 'deleteoldrdn: ', $deleteoldrdn,"\n";
         my $ns = $entry->get_value('newsuperior', asref => 1);
-        $res &&= print _write_attr('newsuperior',$ns,$wrap,$lower) if defined $ns;
+        $res &&= _write_attr('newsuperior',$ns,$wrap,$lower) if defined $ns;
         next;
       }
 
@@ -484,23 +543,19 @@ sub _write_entry {
         }
         my $i = 0;
         while ($i < @$chg) {
-	  $res &&= print "-\n" if $dash++;
+	  $res &&= print "-\n"  if (!$self->{'version'} && $dash++);
           my $attr = $chg->[$i++];
           my $val = $chg->[$i++];
           $res &&= print $type,": ",$attr,"\n";
           $res &&= _write_attr($attr,$val,$wrap,$lower);
+	  $res &&= print "-\n"  if ($self->{'version'});
         }
       }
     }
 
     else {
-      if ($self->{write_count}++) {
-	$res &&= print "\n";
-      }
-      else {
-        $res &&= print "version: $self->{version}\n\n"
-          if defined $self->{version};
-      }
+      $res &&= $self->write_version()  unless $self->{write_count}++;
+      $res &&= print "\n";
       $res &&= _write_dn($entry->dn,$self->{'encode'},$wrap);
       $res &&= _write_attrs($entry,$wrap,$lower,$sort);
     }
@@ -547,6 +602,12 @@ sub done {
   $res;
 }
 
+sub handle {
+  my $self = shift;
+
+  return $self->{fh};
+}
+
 my %onerror = (
   'die'   => sub {
                 my $self = shift;
@@ -603,8 +664,8 @@ sub current_lines {
 
 sub version {
   my $self = shift;
-  return $self->{version} unless @_;
-  $self->{version} = shift;
+  return $self->{'version'} unless @_;
+  $self->{'version'} = shift || 0;
 }
 
 sub next_lines {
