@@ -1,4 +1,4 @@
-# Copyright (c) 1997-2000 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 1997-2004 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
@@ -12,7 +12,7 @@ use Net::LDAP::Filter;
 use Net::LDAP::Constant qw(LDAP_SUCCESS LDAP_DECODING_ERROR);
 
 @ISA = qw(Net::LDAP::Message);
-$VERSION = "0.09";
+$VERSION = "0.11";
 
 
 sub first_entry { # compat
@@ -37,12 +37,13 @@ sub decode {
     if exists $result->{protocolOp}{searchResDone};
 
   my $data;
+  @{$self}{qw(controls ctrl_hash)} = ($result->{controls}, undef);
 
   if ($data = delete $result->{protocolOp}{searchResEntry}) {
 
     my $entry = Net::LDAP::Entry->new;
 
-    $entry->decode($data)
+    $entry->decode($data, raw => $self->{raw})
       or $self->set_error(LDAP_DECODING_ERROR,"LDAP decode error")
      and return;
 
